/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the ARP data structures */
#include "arp.h"
#include "udp.h"
#include <oskit/net/ether.h>

/* Define the UDP data structures */
#include "pxe_udp.h"

/* Define the PXE UNDI data structures */
#include "pxe_undi.h"

/* Define the PXENV+ data structures */
#include "pxenv.h"

#include "nilo.h"

static unsigned char ether_bcast[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

/* The arptable is maintained in network order. */
arptable_t arptable[MAX_ARP];

void
add_to_cache(ip4_t addr, u_int8_t *hwaddr, enum arp_entry a)
{
    NILO_TRACE_FUNCTION;
    
    arptable[a].ipaddr.num = addr.num;
    NILO_TRACE_MSG ("Adding IP #%s with MAC addr %s.\r\n",
    		    _inet_ntoa(addr.num), hexify_buffer (hwaddr, 6));
//    memcpy(&arptable[a].ipaddr.num, &addr, sizeof(addr));
    memcpy(arptable[a].hwaddr, hwaddr, sizeof(arptable[0].hwaddr));
    
    NILO_TRACE_FUNCTION_RETURN;
}

u_int8_t *
lookup_arp(ip4_t addr)
{
    int i;
    NILO_TRACE_FUNCTION;
    
    //    if (addr.num == INADDR_BROADCAST)
    //        return (eth_broadcast);
    for (i=0; i< MAX_ARP; i++) {
      if (addr.num == arptable[i].ipaddr.num) {
	NILO_TRACE_MSG ("Found MAC addr %s for ip #%s.\r\n",
			hexify_buffer(&arptable[i].hwaddr, 6),
			_inet_ntoa(addr.num));
	if (memcmp(arptable[1].hwaddr, "\0\0\0\0\0\0", 6) == 0)
	  return 0;
	else
	  return (arptable[i].hwaddr);
      }
    }
    return 0;
    NILO_TRACE_FUNCTION_RETURN;
}

pxenv_ret_code_t
fill_arp_entry (enum arp_entry arpentry)
{
    ip4_t destip;
    u_int32_t temp, i;
    static int32_t timeout = 0;   
    struct arprequest arpreq;
    pxenv_undi_tbd_t undi_tbd;
    pxenv_undi_transmit_t unditrans;
    pxenv_udp_read_t udpread;
    arp_t arp_reply;

    NILO_TRACE_FUNCTION;

    /* See if we have a hardware address, to go with the IP number */
    for (i = 0; i<MAX_ARP; i++) {
        if (arptable[arpentry].hwaddr[i])
            break;
    }
    
    /* Need to do arp request cause we don't have the mac addr */
    if (i == ETHER_ADDR_SIZE) {
        client_info->client_state = ARPING;
        arpreq.hwtype = htons(1);
        arpreq.protocol = htons(ETHERTYPE_IP);
        arpreq.hwlen = ETHER_ADDR_SIZE;
        arpreq.protolen = 4;
        arpreq.opcode = htons(ARPOP_REQUEST);

        /* setup the Client entry */
        memcpy(arpreq.shwaddr, arptable[ARP_CLIENT].hwaddr,
               ETHER_ADDR_SIZE);
        temp =  arptable[ARP_CLIENT].ipaddr.num;
        memcpy(&arpreq.sipaddr, &temp, 4);

        /* setup the Server entry */
        temp = arptable[arpentry].ipaddr.num;
        memcpy (&arpreq.tipaddr, &temp, sizeof(temp));
        memset((char *)arpreq.thwaddr, 0, ETHER_ADDR_SIZE);
#ifdef NILO_USE_TRACING
        dump_arp_table (&arptable);
        dump_arp_packet (&arpreq);
#endif
        
        timeout = currticks() + DEFAULT_TIMEOUT;

     unditrans.protocol = P_ARP;
     /* if zero, use the dest_addr. If 1, the adapator supplies the
        broadcast address. */
     unditrans.xmit_flags = XMT_BROADCAST;
     /* the hardware address of the destination media. If this isn't
        known, this field can be zero, and xmit_flags must be XMT_BROADCAST. */
//     memset (unditrans.dest_addr, 0, sizeof(unditrans.dest_addr));
     memcpy (unditrans.dest_addr, ether_bcast, sizeof(ether_bcast));
     undi_tbd.length = sizeof(struct arprequest);
     undi_tbd.xmit_buffer = (addr32_t *)&arpreq;
     unditrans.tbd_addr = &undi_tbd;
     
     if (call_pxe_api (PXENV_UNDI_TRANSMIT, &unditrans) == PXENV_EXIT_FAILURE) {
         panic ("Couldn't transmit a packet on using the UNDI.\r\n");
         return (PXENV_EXIT_FAILURE);
     }

     /* Sigh, spin here till the arp table is filled. If we don't, then
      a later transmition to this IP will fail cause there is no MAC
      addr. */
     while (lookup_arp(arptable[arpentry].ipaddr) == 0) {
       NILO_TRACE_MSG ("Wating for the ARP reply...\r\n");
       udpread.dest_ip.num = UDP_READ_ANY_IP;
       udpread.d_port = UDP_READ_ANY_PORT;
       udpread.buffer = (addr32_t *)&arp_reply;
       udpread.buffer_size = sizeof(arp_t);
       if (call_pxe_api (PXENV_UDP_READ, &udpread) == PXENV_EXIT_FAILURE) {
	 NILO_TRACE_MSG ("Couldn't read a UDP packet.\r\n");
	 timeout = currticks() + 16;
	 while (timeout > currticks()) {}
       }
	 timeout = currticks() + 16;
	 while (timeout > currticks()) {}
     }
#if 0
       memcpy (&arp_reply, udpread.buffer, udpread.buffer_size);
       if (ntohs(arp_reply.hwtype) == 1) {
	 NILO_TRACE_MSG ("Got an ARP packet\r\n");
	 memcpy(&temp, arp_reply.sipaddr, sizeof(temp));
	 add_to_cache((ip4_t)temp, (u_int8_t*)&arp_reply.shwaddr,
		      ARP_SERVER);
	 client_info->client_state = ARP_DONE;
#ifdef NILO_USE_TRACING
        dump_arp_packet (&arp_reply);
        dump_arp_table (&arptable);
#endif
	//        if (client_info->client_state == ARPING)
     }
#endif /* 0 */
     client_info->client_state = ARPING;
     
     //     if (client_info->client_state != ARP_DONE)
     //  NILO_TRACE_MSG ("Timed out getting ARP reply\n");
#ifdef NILO_USE_TRACING
        dump_arp_table (&arptable);
#endif     
    }    

    NILO_TRACE_FUNCTION_RETURN;
}
