/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

#include <oskit/machine/asm.h>

#define KERN_CODE_SEG   0x08
#define KERN_DATA_SEG   0x10
#define REAL_MODE_SEG   0x18
#define REAL_MODE_DSEG  0x20
#define CR0_PE          1
#define RELOC		0x0	/* FIXME: this is 0x98000 for etherboot */
        .file   "jump.S"
.text
        .align 4
/*
 *      %eax    entry point
 *      %esi    where kernel is
 *      %edi    where kernel wants to be
 *      %ecx    how big kernel is
 *      %ebx    addr of multiboot_info struct
 *      %edx    segment we should switch to
 */
ENTRY(do_boot)
        /* Set all the data segs to point to %dx. */
        movw    %dx,%ds
        movw    %dx,%es
        movw    %dx,%fs
        movw    %dx,%gs
        movw    %dx,%ss

        rep
        movsb

        jmp     *%eax
ENTRY(do_boot_end)

/*       usage: jumpto(execaddr, location, dhcpaddr);
 *                       4,6     8,10            12,14
 */
	.globl jumpto
        .type    jumpto,@function
jumpto:
	pushl	%ebp
	movl	%esp,%ebp
	pushl	%ebx
	pushl   %esi
	pushl	%edi
	
	movl    8(%ebp),%eax	  /* get the execaddr off the stack */
	movl	%eax,execaddr	  /* stash this away till we need it */
	movl	12(%ebp),%ebx	  /* get the location off the stack */
	movl	16(%ebp),%ecx	  /* get the dhcp data off the stack */
//        movw    %eax,%es	  /* get the execution pointer into ES:BX */
//        movw    %ecx,%ds	  /* get the header pointer into DS:SI */
        call    _prot_to_real
        .code16
        pushl   %ecx            /* dhcp record */
        pushl   %ebx            /* file header */
        movl    $((RELOC<<12)+(1f-RELOC)),%eax
        pushl   %eax
        addr32
        ljmp    execaddr-RELOC

execaddr:
        .long   0

/* _real_to_prot - Go from REAL mode to Protected Mode */
        .globl  _real_to_prot
_real_to_prot:
        .code16
        cli
        cs
        addr32
        lgdt    gdtarg-RELOC
        movl    %cr0,%eax
        orl     $CR0_PE,%eax
        movl    %eax,%cr0               /* turn on protected mode */

        /* flush prefetch queue, and reload %cs:%eip */
        data32
        ljmp    $KERN_CODE_SEG,$1f-RELOC
1:
        .code32
        /* reload other segment registers */
        movl    $KERN_DATA_SEG,%eax
        movw    %ax,%ds
        movw    %ax,%es
        movw    %ax,%ss
        addl    $RELOC,%esp             /* Fix up stack pointer */
        xorl    %eax,%eax
        movw    %ax,%fs
        movw    %ax,%gs
        popl    %eax                    /* Fix up return address */
        addl    $RELOC,%eax
        pushl   %eax
        ret

/* _prot_to_real - Go from Protected Mode to REAL Mode. */
        .globl  _prot_to_real
_prot_to_real:
        .code32
//        popl    %eax
//        subl    $RELOC,%eax             /* Adjust return address */
//        pushl   %eax
// <_prot_to_real+7>:	    sub    $0x98000,%esp
//        subl    $RELOC,%esp             /* Adjust stack pointer */
// <_prot_to_real+13>:	    ljmp   $0x18,$0x26aaef
//	        ljmp    $REAL_MODE_SEG,$1f-RELOC /* jump to a 16 bit segment */
	ljmp    $REAL_MODE_SEG,$1f
1:
        .code16
        movw    $REAL_MODE_DSEG,%ax
        movw    %ax,%ds
        movw    %ax,%ss
        movw    %ax,%es
        movw    %ax,%fs
        movw    %ax,%gs

        /* clear the PE bit of CR0 */
        movl    %cr0,%eax
        andl    $0!CR0_PE,%eax
        movl    %eax,%cr0

        /* make intersegment jmp to flush the processor pipeline
         * and reload %cs:%eip (to clear upper 16 bits of %eip).
         */
        data32
        ljmp    $(RELOC)>>4,$2f-RELOC
2:
        /* we are in real mode now
         * set up the real mode segment registers : %ds, $ss, %es
         */
        movw    %cs,%ax
        movw    %ax,%ds
        movw    %ax,%es
        movw    %ax,%ss
        sti
        data32
        ret             /* There is a 32 bit return address on the stack */
        .code32

/**************************************************************************
GLOBAL DESCRIPTOR TABLE
**************************************************************************/
        .align  4
_gdt:
gdtarg:
        .word   0x27                    /* limit */
        .long   _gdt                    /* addr */
        .byte   0,0

_pmcs:
        /* 32 bit protected mode code segment */
        .word   0xffff,0
        .byte   0,0x9f,0xcf,0

_pmds:
        /* 32 bit protected mode data segment */
        .word   0xffff,0
        .byte   0,0x93,0xcf,0

        /* 16 bit real mode code segment */
        .word   0xffff,(RELOC&0xffff)
        .byte   (RELOC>>16),0x9b,0x00,(RELOC>>24)

        /* 16 bit real mode data segment */
        .word   0xffff,(RELOC&0xffff)
        .byte   (RELOC>>16),0x93,0x00,(RELOC>>24)

        .align  4

        .data
