/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __TAGGED_H__
#define __TAGGED_H__

/* This is the include generated by configuring */
#include "niloconfig.h"

/* Define the PXE types */
#include "pxe_types.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define some structures used by tftp loader */

#define	TAG_MAGIC	0x1B031336L

#define LOAD_ABSOLUTE 0                 /* load address is an absolute 32 bit number */
#define LOAD_SUB_ADDR 1                 /* last writeable location in
                                           memory. Thus 1 would be the last
                                           location one could write in
                                           memory. */ 

#define LOAD_ADD_ADDR 2                 /* add the load address to the
                                           location one past the last
                                           byte memory area required by the
                                           last image loaded. If the first
                                           image, then add to 512 plus the
                                           location where the 512 bytes were
                                           placed  */ 
#define LOAD_SUB_START 3                /* load address is subtracted from the
                                           start of the last image loaded. If
                                           the first image, then subtract from
                                           the start of where the 512 bytes
                                           were placed  */


struct imgheader {
    unsigned long magic;
    unsigned nlen:4;                    /* non vendor length */
    unsigned vlen:4;                    /* vendor length */
    unsigned bootret:1;                 /* Set if boot image can return */
    unsigned reserved:23;               /* reserved for future use */
    segoff16_t location;                /* bx, ds */
    segoff16_t execaddr;                /* ip, cs */
};

union infoblock {
    unsigned short s[256];
    unsigned long l[128];
    struct imgheader i;
};

struct segheader {
    unsigned nlen:4;
    unsigned vlen:4;
    unsigned char vendorinfo;
    unsigned char reserved;
    unsigned flags:2;
    unsigned endmark:1;
    unsigned undefined:5;
    unsigned long loadaddr;
    unsigned long imglength;
    unsigned long memlength;
};

/* FIXME: we should include mknbi.h, but for now we just put these here
 *
 * Ramdisk images can be loaded in three different ways:
 *  RD_AUTO  --  The ramdisk image is loaded at either 0x100000 or right
 *               behind the kernel image; at run-time it will be moved
 *               to the end of physical memory or right before 0x1000000.
 *               This technique guarantees maximum compatibility with
 *               broken BIOS versions without making assumptions on the
 *               way, the BOOT-Prom determined the end of memory. The
 *               drawback is, that moving the image costs some extra time.
 *  RD_EOM   --  The ramdisk image is loaded right before the end of
 *               physical memory. It is the BOOT-Proms responsibility to do
 *               this reliably. This might cause problems with some ancient
 *               BIOS versions, if the computer is equipped with more then
 *               16M of main memory.
 *  RD_FIXED --  The user provided a fixed load address for the ramdisk
 *               image. This entails possible compatibility restrictions
 *               when moving the image from one client to another. OTOH,
 *               it could be the final solution if everything else fails...
 */
#define RD_AUTO         0
#define RD_EOM          1
#define RD_FIXED        2

/* These are the Linux specific Vendor Tags of the segment header */
#define VTAG_BOOT_LOADER        0x10    /* primary boot loader */
#define VTAG_CMD_LINE           0x11    /* command line */
#define VTAG_FLOPPY_SECTOR      0x12    /* floppy boot sector */
#define VTAG_KERNEL_SETUP       0x13    /* kernel setup */
#define VTAG_KERNEL_IMAGE       0x14    /* kernel image */
#define VTAG_RAMDISK            0x15    /* ramdisk image */

extern void handle_block(int blockno, union infoblock *data);

#endif /* __TAGGED_H__ */


